"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControllerFacility = exports.LoggerFacility = exports.ForceHttpsMiddleware = exports.RestfulApiFacility = exports.WebApiFacility = void 0;
const tslib_1 = require("tslib");
const cors_1 = tslib_1.__importDefault(require("@koa/cors"));
const core_1 = require("@plumier/core");
const generic_controller_1 = require("@plumier/generic-controller");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const koa_body_1 = tslib_1.__importDefault(require("koa-body"));
const qs_1 = tslib_1.__importDefault(require("qs"));
function createQuery(queryString, extra) {
    const parsed = qs_1.default.parse(queryString);
    if (!extra)
        return parsed;
    const raw = Object.keys(extra).reduce((result, q) => {
        result[q] = extra[q];
        return result;
    }, parsed);
    return new Proxy(raw, {
        get: (target, name) => {
            for (const key in target) {
                if (key.toLowerCase() === name.toString().toLowerCase())
                    return target[key];
            }
            return target[name];
        }
    });
}
function copyDescriptor(dest, src) {
    Object.getOwnPropertyNames(src).forEach((name) => {
        var descriptor = Object.getOwnPropertyDescriptor(src, name);
        Object.defineProperty(dest, name, descriptor);
    });
}
/**
 * Preset configuration for building rest. This facility contains:
 *
 * parameter binder
 *
 * body parser: koa-body
 *
 * cors: @koa/cors
 */
class WebApiFacility extends core_1.DefaultFacility {
    constructor(opt) {
        super();
        this.opt = opt;
    }
    async generateRoutes(app) {
        const { controller } = app.config;
        const controllers = await (0, core_1.generateRoutes)(controller, Object.assign(Object.assign({}, app.config), this.opt));
        const generics = await (0, generic_controller_1.generateGenericControllerRoutes)(controller, Object.assign(Object.assign({}, app.config), this.opt));
        return [...controllers, ...generics];
    }
    setup(app) {
        var _a;
        const option = Object.assign({}, this.opt);
        if ((_a = option.forceHttps) !== null && _a !== void 0 ? _a : (0, core_1.toBoolean)(process.env.PLUM_FORCE_HTTPS || "__none")) {
            app.use(new ForceHttpsMiddleware());
        }
        app.koa.use((0, koa_body_1.default)(option.bodyParser));
        if (typeof option.cors !== "boolean" && option.cors) {
            app.koa.use((0, cors_1.default)(option.cors));
        }
        else if (option.cors) {
            app.koa.use((0, cors_1.default)());
        }
        if (option.dependencyResolver)
            app.set({ dependencyResolver: option.dependencyResolver });
        if (option.controller)
            app.set({ controller: option.controller });
        if (option.trustProxyHeader)
            app.set({ trustProxyHeader: option.trustProxyHeader });
        // update query 
        copyDescriptor(app.koa.request, {
            addQuery: function (query) {
                this._extraQuery = query;
            },
            get query() {
                const str = this.querystring;
                const cache = this._querycache = this._querycache || {};
                return cache[str] || (cache[str] = createQuery(str, this._extraQuery));
            }
        });
    }
}
exports.WebApiFacility = WebApiFacility;
/**
 * Preset configuration for building restful style api. This facility contains:
 *
 * parameter binder
 *
 * body parser: koa-body
 *
 * cors: @koa/cors
 *
 * default response status: { get: 200, post: 201, put: 204, delete: 204 }
 */
class RestfulApiFacility extends WebApiFacility {
    setup(app) {
        super.setup(app);
        app.set({ responseStatus: { post: 201, put: 204, delete: 204 } });
    }
}
exports.RestfulApiFacility = RestfulApiFacility;
class ForceHttpsMiddleware {
    async execute(i) {
        const req = i.ctx.request;
        if (req.protocol === "http")
            return core_1.response.redirect(`https://${req.hostname}${req.originalUrl}`);
        else
            return i.proceed();
    }
}
exports.ForceHttpsMiddleware = ForceHttpsMiddleware;
class LoggerFacility extends core_1.DefaultFacility {
    setup(app) {
        app.use(new LoggerMiddleware());
    }
}
exports.LoggerFacility = LoggerFacility;
class LoggerMiddleware {
    async execute(i) {
        const log = (msg) => {
            if (i.ctx.config.mode === "debug")
                console.log(msg);
        };
        const start = new Date();
        const getTime = () => `${(new Date().getTime() - start.getTime())}ms`;
        try {
            const result = await i.proceed();
            log(chalk_1.default.green(`${i.ctx.method} ${result.status || 200} ${i.ctx.url} ${getTime()}`));
            return result;
        }
        catch (e) {
            if (e instanceof core_1.HttpStatusError) {
                log(chalk_1.default.yellow(`${i.ctx.method} ${e.status} ${i.ctx.url} ${getTime()}`));
                if (e.message)
                    log(chalk_1.default.yellow(e.message));
            }
            else {
                log(chalk_1.default.red(`${i.ctx.method} 500 ${i.ctx.url} ${getTime()}`));
                log(chalk_1.default.red(e.stack));
            }
            throw e;
        }
    }
}
class ControllerFacility extends core_1.DefaultFacility {
    constructor(option) {
        super();
        this.option = option;
    }
    setup(app) {
        app.set({ controller: "__UNSET__" });
    }
    async generateRoutes(app) {
        const { rootDir } = app.config;
        const controller = this.option.controller;
        const controllers = await (0, core_1.generateRoutes)(controller, Object.assign(Object.assign({}, app.config), this.option));
        const generics = await (0, generic_controller_1.generateGenericControllerRoutes)(controller, Object.assign(Object.assign({}, app.config), this.option));
        return [...controllers, ...generics];
    }
}
exports.ControllerFacility = ControllerFacility;
